"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useSorting = void 0;
const react_1 = require("react");
const defaultCurrentSort = {};
const defaultSortQueryString = "sort";
const composeSortedFieldsForRest = (masterFactory, sortedFields) => {
    const sortedFieldsForRest = {};
    Object.keys(sortedFields).forEach(field => {
        var _a;
        const childProps = (_a = masterFactory.childrenProps) === null || _a === void 0 ? void 0 : _a.find(({ propertyName }) => propertyName === field);
        if (typeof childProps.restPropertyName === "string") {
            sortedFieldsForRest[childProps.restPropertyName] = sortedFields[field];
        }
        else if (typeof childProps.restPropertyName === "object") {
            Object.keys(childProps.restPropertyName).forEach(key => (sortedFieldsForRest[childProps.restPropertyName[key]] = sortedFields[field]));
        }
        else {
            sortedFieldsForRest[childProps.propertyName] = sortedFields[field];
        }
    });
    return sortedFieldsForRest;
};
exports.useSorting = (config = {}) => {
    const [sortedFields, setSortedFields] = react_1.useState(config.currentSortedFields || defaultCurrentSort);
    const [masterFactory, setMasterFactory] = react_1.useState();
    const [dataResourceUrl, setDataResourceUrl] = react_1.useState();
    const [queryString, setQueryString] = react_1.useState();
    const sortQueryString = config.sortQueryString || defaultSortQueryString;
    const orderQueryString = config.orderQueryString || sortQueryString;
    const safeSetMasterFactory = react_1.useCallback((currentMasterFactory) => {
        if (JSON.stringify(currentMasterFactory) !== JSON.stringify(masterFactory)) {
            setMasterFactory(currentMasterFactory);
        }
    }, [masterFactory]);
    react_1.useEffect(() => {
        setSortedFields(config.currentSortedFields);
    }, [config.currentSortedFields]);
    react_1.useEffect(() => {
        if (masterFactory && sortedFields) {
            let queryString = "";
            const sortedFieldsForRest = composeSortedFieldsForRest(masterFactory, sortedFields);
            if (sortQueryString === orderQueryString) {
                Object.keys(sortedFieldsForRest).forEach(field => {
                    queryString += `${queryString !== "" ? "&" : ""}${sortQueryString}=${field},${sortedFieldsForRest[field]}`;
                });
            }
            else {
                if (Object.keys(sortedFieldsForRest).length > 0) {
                    queryString = `${sortQueryString}=${Object.keys(sortedFieldsForRest)
                        .map(field => field)
                        .join(",")}&${orderQueryString}=${Object.keys(sortedFieldsForRest)
                        .map(field => sortedFieldsForRest[field])
                        .join(",")}`;
                }
            }
            setQueryString(queryString);
        }
    }, [masterFactory, sortedFields]);
    return {
        sortingDataResourceUrl: dataResourceUrl
            ? `${dataResourceUrl}${queryString ? `${dataResourceUrl.includes("?") ? "&" : "?"}${queryString}` : ""}`
            : null,
        sortedFields,
        setSortedFields,
        setMasterFactory: safeSetMasterFactory,
        setDataResourceUrl,
    };
};
