"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MasterFactoryContext = void 0;
const react_1 = __importStar(require("react"));
const react_bootstrap_1 = require("react-bootstrap");
const styled_components_1 = __importDefault(require("styled-components"));
const axios_1 = __importDefault(require("axios"));
const path_to_regexp_1 = require("path-to-regexp");
const octicons_react_1 = require("@primer/octicons-react");
const FormContainer_1 = __importDefault(require("./form/FormContainer"));
const Header_1 = __importDefault(require("./header/Header"));
const RemoteDataTable_1 = __importDefault(require("../datatable/remote/RemoteDataTable"));
const Message_1 = __importDefault(require("../message/Message"));
const ModalContainer_1 = __importDefault(require("../modal/ModalContainer"));
const Confirm_1 = __importDefault(require("../modal/confirm/Confirm"));
exports.MasterFactoryContext = react_1.createContext({});
const Actions = styled_components_1.default.div `
  width: 75px;
`;
const composeColumns = (childrenProps) => {
    const result = {};
    childrenProps
        .filter(({ showOnDataTable }) => showOnDataTable)
        .forEach(({ label, propertyName }) => {
        result[propertyName] = label;
    });
    return result;
};
const compileFromUrl = (url, datum) => {
    const { origin, pathname } = new URL(url);
    const buildPath = path_to_regexp_1.compile(pathname, { encode: encodeURIComponent });
    return `${origin}${buildPath(datum)}`;
};
const Master = (props) => {
    const [editableDatum, setEditableDatum] = react_1.useState();
    const [selectedData, setSelectedData] = react_1.useState([]);
    const remoteDataTable = react_1.useRef();
    const formModal = react_1.useRef();
    const confirm = react_1.useRef();
    const message = react_1.useRef();
    const childrenProps = react_1.Children.map(props.children, (field) => field.props);
    const tableCellComponents = react_1.Children.map(props.children, (field) => ({
        propertyName: field.props.propertyName,
        TableCellComponent: field.type.TableCellComponent,
    }));
    const openForm = react_1.useCallback((datum) => {
        formModal.current.show();
        if (typeof datum.preventDefault !== "function") {
            setEditableDatum(datum);
        }
    }, []);
    const closeForm = react_1.useCallback(() => {
        setEditableDatum(null);
    }, []);
    const onSubmit = react_1.useCallback((fields) => __awaiter(void 0, void 0, void 0, function* () {
        const request = !editableDatum
            ? axios_1.default[props.newInstanceFetchingMethod]
            : axios_1.default[props.updateInstanceFetchingMethod];
        try {
            yield request(!editableDatum
                ? props.newInstanceFetchingResourceUrl
                : compileFromUrl(props.updateInstanceFetchingResourceUrl, editableDatum), fields);
            formModal.current.hide();
            remoteDataTable.current.reload();
            message.current.show({
                content: !editableDatum ? props.successToAddMessage : props.successToUpdateMessage,
                type: "success",
            });
            setEditableDatum(null);
        }
        catch (e) {
            message.current.show({
                content: e.message || e,
                type: "danger",
            });
            !editableDatum ? props.onNewInstanceRequestError(e) : props.onUpdateInstanceRequestError(e);
        }
    }), [editableDatum, props.newInstanceFetchingMethod, props.updateInstanceFetchingMethod]);
    const deleteData = react_1.useCallback((data) => {
        if (data.length > 0) {
            confirm.current.show({
                title: props.deleteConfirmationTitle,
                message: data.length === 1 ? props.deleteConfirmationMessage.singular : props.deleteConfirmationMessage.plural,
                type: "danger",
                onYesButtonClick: () => __awaiter(void 0, void 0, void 0, function* () {
                    try {
                        const promises = data.map(datum => axios_1.default.delete(compileFromUrl(props.deleteInstanceFetchingResourceUrl, datum)));
                        yield Promise.all(promises);
                        remoteDataTable.current.reload();
                        message.current.show({
                            content: props.successToDeleteMessage,
                            type: "success",
                        });
                    }
                    catch (e) {
                        message.current.show({
                            content: e.message || e,
                            type: "danger",
                        });
                        props.onDeleteInstanceRequestError(e);
                    }
                }),
            });
        }
    }, []);
    const deleteAllSelectedData = react_1.useCallback(() => __awaiter(void 0, void 0, void 0, function* () {
        deleteData(selectedData);
        setSelectedData([]);
    }), [selectedData]);
    const customRenderCell = react_1.useCallback((datum, propertyName) => {
        var _a;
        const childProps = childrenProps.find(childProps => childProps.propertyName === propertyName);
        datum = JSON.parse(JSON.stringify(datum));
        if (childProps.customInboundRestParser) {
            datum[propertyName] = childProps.customInboundRestParser(datum[propertyName]);
        }
        if (childProps.customRenderCell) {
            return childProps.customRenderCell(datum);
        }
        else {
            const TableCellComponent = (_a = tableCellComponents.find(tableCellComponent => tableCellComponent.propertyName === propertyName)) === null || _a === void 0 ? void 0 : _a.TableCellComponent;
            if (TableCellComponent) {
                return (react_1.default.createElement(TableCellComponent, Object.assign({}, childProps, { datum: datum, propertyName: propertyName, restPropertyName: childProps.restPropertyName })));
            }
        }
        return null;
    }, []);
    const renderAction = react_1.useCallback((datum) => {
        if (props.customRenderAction) {
            const action = props.customRenderAction(datum);
            if (action) {
                return action;
            }
        }
        return (react_1.default.createElement(Actions, { className: "text-center mx-auto" },
            react_1.default.createElement(react_bootstrap_1.Button, { className: "mr-1", variant: "outline-primary", size: "sm", onClick: () => openForm(datum) },
                react_1.default.createElement(octicons_react_1.PencilIcon, { verticalAlign: "middle", size: 16 })),
            react_1.default.createElement(react_bootstrap_1.Button, { variant: "outline-danger", size: "sm", onClick: () => deleteData([datum]) },
                react_1.default.createElement(octicons_react_1.TrashcanIcon, { verticalAlign: "middle", size: 16 }))));
    }, [props.customRenderAction]);
    const onSelectedDataChange = react_1.useCallback((data) => setSelectedData(data), []);
    return (react_1.default.createElement(exports.MasterFactoryContext.Provider, { value: { childrenProps } },
        react_1.default.createElement(react_bootstrap_1.Container, null,
            react_1.default.createElement(Header_1.default, { title: props.title, deleteSelectedButtonLabel: props.deleteSelectedButtonLabel, addNewButtonLabel: props.addNewButtonLabel, showDeleteSelectedButton: selectedData.length > 0, onDeleteSelectedButtonClick: deleteAllSelectedData, onAddNewButtonClick: openForm }),
            react_1.default.createElement(RemoteDataTable_1.default, { ref: remoteDataTable, dataResourceHeaders: props.dataFetchingResourceHeaders, dataResourceUrl: props.dataFetchingResourceUrl, dataPropertyPath: props.dataFetchingPropertyPath, pagination: props.dataFetchingPagination, sorting: props.dataFetchingSorting, selectable: true, noDataAvailableLabel: props.noDataAvailableLabel, actionLabel: props.actionLabel, sortLabel: props.sortLabel, sortAscLabel: props.sortAscLabel, sortDescLabel: props.sortDescLabel, addNewFilterLabel: props.addNewFilterLabel, columns: composeColumns(childrenProps), customRenderCell: customRenderCell, renderAction: renderAction, interceptResponse: props.interceptDataFetchingResponse, onPageChange: props.onDataFetchingPageChange, onSortedFieldsChange: props.onDataFetchingSortedFieldsChange, onError: props.onDataFetchingRequestError, onSelectedDataChange: onSelectedDataChange }),
            react_1.default.createElement(ModalContainer_1.default, { ref: formModal, size: "lg", title: !editableDatum ? props.addNewFormTitle : props.updateFormTitle, onHide: closeForm },
                react_1.default.createElement(FormContainer_1.default, { datum: editableDatum, loadingLabel: props.loadingLabel, saveButtonLabel: props.saveButtonLabel, onSubmit: onSubmit }, props.children)),
            react_1.default.createElement(Confirm_1.default, { ref: confirm }),
            react_1.default.createElement(Message_1.default, { ref: message }))));
};
const defaultProps = {
    children: [],
    loadingLabel: "Loading",
    addNewFormTitle: "Add New Item",
    updateFormTitle: "Update Item",
    deleteConfirmationTitle: "Delete Item",
    deleteConfirmationMessage: {
        singular: "Are you sure to delete this item?",
        plural: "Are you sure to delete these items?",
    },
    successToAddMessage: "Success to add new instance",
    successToUpdateMessage: "Success to update instance",
    successToDeleteMessage: "Success to delete instance(s)",
    newInstanceFetchingMethod: "post",
    updateInstanceFetchingMethod: "put",
    onNewInstanceRequestError: () => { },
    onUpdateInstanceRequestError: () => { },
    onDeleteInstanceRequestError: () => { },
};
Master.defaultProps = defaultProps;
exports.default = Master;
