"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FormFactoryContext = void 0;
const react_1 = __importStar(require("react"));
const react_bootstrap_1 = require("react-bootstrap");
class FormFactory {
    constructor() {
        this.fields = {};
        this.observers = {};
        this.inboundRestParsers = {};
        this.outboundRestParsers = {};
    }
    composeCustomRestValue(restPropertyName, value) {
        const result = {};
        Object.keys(value).forEach(key => value && restPropertyName && (result[restPropertyName[key]] = value[key]));
        return result;
    }
    setRestParser(propertyName, inboundRestParser, outboundRestParser) {
        if (inboundRestParser) {
            this.inboundRestParsers[propertyName] = inboundRestParser;
        }
        if (outboundRestParser) {
            this.outboundRestParsers[propertyName] = outboundRestParser;
        }
    }
    observe(propertyName, observer) {
        this.observers[propertyName] = observer;
    }
    setField(propertyName, restPropertyName, value) {
        this.fields[propertyName] = {
            value,
            restValue: typeof restPropertyName === "string" ? value : this.composeCustomRestValue(restPropertyName, value),
        };
        if (this.observers[propertyName]) {
            this.observers[propertyName](value);
        }
    }
    getField(propertyName) {
        var _a;
        return (_a = this.fields[propertyName]) === null || _a === void 0 ? void 0 : _a.value;
    }
    getFields() {
        const result = {};
        Object.keys(this.fields).forEach(propertyName => { var _a; return (result[propertyName] = (_a = this.fields[propertyName]) === null || _a === void 0 ? void 0 : _a.value); });
        return result;
    }
    clearFields() {
        this.fields = {};
    }
    getRestFields() {
        let result = {};
        Object.keys(this.fields).forEach(propertyName => {
            var _a, _b, _c, _d, _e;
            if (((_a = this.fields[propertyName]) === null || _a === void 0 ? void 0 : _a.restValue) === ((_b = this.fields[propertyName]) === null || _b === void 0 ? void 0 : _b.value)) {
                result[propertyName] = this.outboundRestParsers[propertyName]
                    ? this.outboundRestParsers[propertyName]((_c = this.fields[propertyName]) === null || _c === void 0 ? void 0 : _c.value)
                    : (_d = this.fields[propertyName]) === null || _d === void 0 ? void 0 : _d.value;
            }
            else {
                let restValue = ((_e = this.fields[propertyName]) === null || _e === void 0 ? void 0 : _e.restValue) || {};
                if (this.outboundRestParsers[propertyName]) {
                    const parsed = this.outboundRestParsers[propertyName](restValue);
                    if (typeof parsed === "object") {
                        restValue = parsed;
                    }
                }
                result = Object.assign(Object.assign({}, result), restValue);
            }
        });
        return JSON.parse(JSON.stringify(result));
    }
    getInboundRestParsers(propertyName) {
        return this.inboundRestParsers[propertyName];
    }
    getOutboundRestParsers(propertyName) {
        return this.outboundRestParsers[propertyName];
    }
}
exports.FormFactoryContext = react_1.createContext(null);
const FormContainer = (props) => {
    const [factory] = react_1.useState(new FormFactory());
    const onSubmit = react_1.useCallback((e) => {
        e.preventDefault();
        props.onSubmit(factory.getRestFields());
    }, [factory]);
    react_1.useEffect(() => {
        if (props.datum) {
            const datum = JSON.parse(JSON.stringify(props.datum));
            react_1.Children.forEach(props.children, (field) => {
                const fieldProps = field.props;
                if (fieldProps.showOnForm) {
                    if (typeof fieldProps.restPropertyName === "string" && datum[fieldProps.restPropertyName] != null) {
                        factory.setField(fieldProps.propertyName, fieldProps.restPropertyName, datum[fieldProps.restPropertyName]);
                    }
                    else if (!fieldProps.restPropertyName && datum[fieldProps.propertyName] != null) {
                        factory.setField(fieldProps.propertyName, fieldProps.restPropertyName || fieldProps.propertyName, datum[fieldProps.propertyName]);
                    }
                    else if (typeof fieldProps.restPropertyName === "object") {
                        const value = {};
                        Object.keys(fieldProps.restPropertyName).forEach(key => (value[key] = datum[fieldProps.restPropertyName[key]]));
                        factory.setField(fieldProps.propertyName, fieldProps.restPropertyName, value);
                    }
                }
            });
        }
        else {
            factory.clearFields();
        }
    }, [props.children, props.datum]);
    react_1.useEffect(() => {
        factory.loadingLabel = props.loadingLabel;
    }, [props.loadingLabel]);
    return (react_1.default.createElement(exports.FormFactoryContext.Provider, { value: factory },
        react_1.default.createElement(react_bootstrap_1.Form, { onSubmit: onSubmit },
            props.children,
            react_1.default.createElement("hr", null),
            react_1.default.createElement(react_bootstrap_1.Button, { variant: "primary", size: "lg", type: "submit", block: true }, props.saveButtonLabel))));
};
const defaultProps = {
    onSubmit: () => { },
    saveButtonLabel: "Save",
};
FormContainer.defaultProps = defaultProps;
exports.default = FormContainer;
