"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useLoadData = exports.loadRemoteData = void 0;
const axios_1 = __importDefault(require("axios"));
const react_1 = require("react");
const Error_1 = require("./Error");
const remoteDataCache = {};
const _loadRemoteData = (resourceUrl, propertyPathName, propertyPathValue, shouldBeAnArray = true) => __awaiter(void 0, void 0, void 0, function* () {
    const response = yield axios_1.default.get(resourceUrl);
    let responseData = response.data;
    if (propertyPathValue) {
        responseData = propertyPathValue.split(".").reduce((parent, propertyName) => parent[propertyName], responseData);
    }
    if ((shouldBeAnArray && Array.isArray(responseData)) || !shouldBeAnArray) {
        return responseData;
    }
    else {
        Error_1.throwRemoteResponseError(propertyPathName, propertyPathValue, responseData);
        return null;
    }
});
exports.loadRemoteData = (resourceUrl, propertyPathName, propertyPathValue, shouldBeAnArray = true, cached = false) => __awaiter(void 0, void 0, void 0, function* () {
    if (cached && remoteDataCache[resourceUrl]) {
        return yield remoteDataCache[resourceUrl];
    }
    const promise = _loadRemoteData(resourceUrl, propertyPathName, propertyPathValue, shouldBeAnArray);
    if (cached) {
        remoteDataCache[resourceUrl] = promise;
    }
    return yield promise;
});
exports.useLoadData = (props, resourceUrl, propertyPathName, propertyPath) => {
    const loadData = react_1.useCallback(() => __awaiter(void 0, void 0, void 0, function* () {
        if (!(props === null || props === void 0 ? void 0 : props.isLoading) && resourceUrl) {
            (props === null || props === void 0 ? void 0 : props.setLoading) && props.setLoading(true);
            const responseData = (yield exports.loadRemoteData(resourceUrl, propertyPathName, propertyPath, true, true));
            (props === null || props === void 0 ? void 0 : props.setLoading) && props.setLoading(false);
            return responseData;
        }
        return null;
    }), [props === null || props === void 0 ? void 0 : props.isLoading, resourceUrl, propertyPath]);
    return { loadData };
};
